# -*- coding: utf-8 -*-
from amplify.agent.containers.nginx.log.error import NginxErrorLogParser
from amplify.agent.util.tail import FileTail
from amplify.agent.context import context
from amplify.agent.containers.abstract import AbstractCollector
from amplify.agent.containers.nginx.config.config import ERROR_LOG_LEVELS


__author__ = "Mike Belov"
__copyright__ = "Copyright (C) 2015, Nginx Inc. All rights reserved."
__credits__ = ["Mike Belov", "Andrei Belov", "Ivan Poluyanov", "Oleg Mamontov", "Andrew Alexeev", "Grant Hulegaard"]
__license__ = ""
__maintainer__ = "Mike Belov"
__email__ = "dedm@nginx.com"


class NginxErrorLogsCollector(AbstractCollector):

    short_name = 'nginx_elog'

    counters = (
        'http.request.buffered',
        #'http.request.malformed',
        #'http.request.failed',
        #'http.response.failed',
        'upstream.response.buffered',
        'upstream.request.failed',
        'upstream.response.failed',
        #'http.ssl_handshake.failed',
        #'upstream.fastcgi.errors',
        #'http.request.limited',
        #'cache.fs.errors',
        #'cache.lock.timeouts',
        #'upstream.health_check.failed',
        #'workers.warn.low_conn'
    )

    def __init__(self, filename=None, level=None, log_format=None, tail=None, **kwargs):
        super(NginxErrorLogsCollector, self).__init__(**kwargs)
        self.filename = filename
        self.level = level
        self.parser = NginxErrorLogParser()
        self.tail = tail if tail is not None else FileTail(filename)

    def collect(self):
        if ERROR_LOG_LEVELS.index(self.level) <= 3:
            self.init_counters()  # set all error counters to 0

        count = 0
        for line in self.tail:
            count += 1
            try:
                error = self.parser.parse(line)
            except:
                context.log.debug('could not parse line %s' % line, exc_info=True)
                error = None

            if error:
                try:
                    self.statsd.incr(error)
                except Exception as e:
                    exception_name = e.__class__.__name__
                    context.log.error('failed to collect error log metrics due to %s' % exception_name)
                    context.log.debug('additional info:', exc_info=True)

        context.log.debug('%s processed %s lines from %s' % (self.object.id, count, self.filename))
